/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.ext.rbvi.chimera;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

import jalview.datamodel.Alignment;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.ColumnSelection;
import jalview.datamodel.Sequence;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignFrame;
import jalview.gui.SequenceRenderer;
import jalview.schemes.JalviewColourScheme;
import jalview.structure.StructureMapping;
import jalview.structure.StructureSelectionManager;

import java.awt.Color;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.testng.annotations.Test;

public class ChimeraXCommandsTest
{

  @Test(groups = { "Functional" })
  public void testBuildColourCommands()
  {

    Map<Object, AtomSpecModel> map = new LinkedHashMap<>();
    ChimeraCommands.addAtomSpecRange(map, Color.blue, 0, 2, 5, "A");
    ChimeraCommands.addAtomSpecRange(map, Color.blue, 0, 7, 7, "B");
    ChimeraCommands.addAtomSpecRange(map, Color.blue, 0, 9, 23, "A");
    ChimeraCommands.addAtomSpecRange(map, Color.blue, 1, 1, 1, "A");
    ChimeraCommands.addAtomSpecRange(map, Color.blue, 1, 4, 7, "B");
    ChimeraCommands.addAtomSpecRange(map, Color.yellow, 1, 8, 8, "A");
    ChimeraCommands.addAtomSpecRange(map, Color.yellow, 1, 3, 5, "A");
    ChimeraCommands.addAtomSpecRange(map, Color.red, 0, 3, 5, "A");
    ChimeraCommands.addAtomSpecRange(map, Color.red, 0, 6, 9, "A");

    // Colours should appear in the Chimera command in the order in which
    // they were added; within colour, by model, by chain, ranges in start order
    String command = new ChimeraXCommands().buildColourCommands(map).get(0);
    assertEquals(
            command,
            "color #0/A:2-5,9-23/B:7|#1/A:1/B:4-7 #0000ff; color #1/A:3-5,8 #ffff00; color #0/A:3-9 #ff0000");
  }

  @Test(groups = { "Functional" })
  public void testBuildSetAttributeCommands()
  {
    /*
     * make a map of { featureType, {featureValue, {residue range specification } } }
     */
    Map<String, Map<Object, AtomSpecModel>> featuresMap = new LinkedHashMap<>();
    Map<Object, AtomSpecModel> featureValues = new HashMap<>();
    
    /*
     * start with just one feature/value...
     */
    featuresMap.put("chain", featureValues);
    ChimeraCommands.addAtomSpecRange(featureValues, "X", 0, 8, 20, "A");
  
    ChimeraXCommands commandGenerator = new ChimeraXCommands();
    List<String> commands = commandGenerator
            .buildSetAttributeCommands(featuresMap);
    assertEquals(1, commands.size());

    /*
     * feature name gets a jv_ namespace prefix
     * feature value is quoted in case it contains spaces
     */
    assertEquals(commands.get(0),
            "setattr #0/A:8-20 res jv_chain 'X' create true");

    // add same feature value, overlapping range
    ChimeraCommands.addAtomSpecRange(featureValues, "X", 0, 3, 9, "A");
    // same feature value, contiguous range
    ChimeraCommands.addAtomSpecRange(featureValues, "X", 0, 21, 25, "A");
    commands = commandGenerator.buildSetAttributeCommands(featuresMap);
    assertEquals(1, commands.size());
    assertEquals(commands.get(0),
            "setattr #0/A:3-25 res jv_chain 'X' create true");

    // same feature value and model, different chain
    ChimeraCommands.addAtomSpecRange(featureValues, "X", 0, 21, 25, "B");
    // same feature value and chain, different model
    ChimeraCommands.addAtomSpecRange(featureValues, "X", 1, 26, 30, "A");
    commands = commandGenerator.buildSetAttributeCommands(featuresMap);
    assertEquals(1, commands.size());
    assertEquals(commands.get(0),
            "setattr #0/A:3-25/B:21-25|#1/A:26-30 res jv_chain 'X' create true");

    // same feature, different value
    ChimeraCommands.addAtomSpecRange(featureValues, "Y", 0, 40, 50, "A");
    commands = commandGenerator.buildSetAttributeCommands(featuresMap);
    assertEquals(2, commands.size());
    // commands are ordered by feature type but not by value
    // so use contains to test for the expected command:
    assertTrue(commands
            .contains(
                    "setattr #0/A:3-25/B:21-25|#1/A:26-30 res jv_chain 'X' create true"));
    assertTrue(commands
            .contains("setattr #0/A:40-50 res jv_chain 'Y' create true"));

    featuresMap.clear();
    featureValues.clear();
    featuresMap.put("side-chain binding!", featureValues);
    ChimeraCommands.addAtomSpecRange(featureValues,
            "<html>metal <a href=\"http:a.b.c/x\"> 'ion!", 0, 7, 15,
            "A");
    // feature names are sanitised to change non-alphanumeric to underscore
    // feature values are sanitised to encode single quote characters
    commands = commandGenerator.buildSetAttributeCommands(featuresMap);
    assertTrue(commands.contains(
            "setattr #0/A:7-15 res jv_side_chain_binding_ '<html>metal <a href=\"http:a.b.c/x\"> &#39;ion!' create true"));
  }

  @Test(groups = { "Functional" })
  public void testColourBySequence_hiddenColumns()
  {
    /*
     * load these sequences, coloured by Strand propensity,
     * with columns 2-4 hidden
     */
    SequenceI seq1 = new Sequence("seq1", "MHRSQSSSGG");
    SequenceI seq2 = new Sequence("seq2", "MVRSNGGSSS");
    AlignmentI al = new Alignment(new SequenceI[] { seq1, seq2 });
    AlignFrame af = new AlignFrame(al, 800, 500);
    af.changeColour_actionPerformed(JalviewColourScheme.Strand.toString());
    ColumnSelection cs = new ColumnSelection();
    cs.addElement(2);
    cs.addElement(3);
    cs.addElement(4);
    af.getViewport().setColumnSelection(cs);
    af.hideSelColumns_actionPerformed(null);
    SequenceRenderer sr = new SequenceRenderer(af.getViewport());
    SequenceI[][] seqs = new SequenceI[][] { { seq1 }, { seq2 } };
    String[] files = new String[] { "seq1.pdb", "seq2.pdb" };
    StructureSelectionManager ssm = new StructureSelectionManager();

    /*
     * map residues 1-10 to residues 21-30 (atoms 105-150) in structures
     */
    HashMap<Integer, int[]> map = new HashMap<>();
    for (int pos = 1; pos <= seq1.getLength(); pos++)
    {
      map.put(pos, new int[] { 20 + pos, 5 * (20 + pos) });
    }
    StructureMapping sm1 = new StructureMapping(seq1, "seq1.pdb", "pdb1",
            "A", map, null);
    ssm.addStructureMapping(sm1);
    StructureMapping sm2 = new StructureMapping(seq2, "seq2.pdb", "pdb2",
            "B", map, null);
    ssm.addStructureMapping(sm2);

    String[] commands = new ChimeraXCommands()
            .colourBySequence(ssm, files, seqs, sr, af.alignPanel);
    assertEquals(1, commands.length);
    String theCommand = commands[0];
    // M colour is #82827d (see strand.html help page)
    assertTrue(theCommand.contains("color #0/A:21|#1/B:21 #82827d"));// #0:21.A|#1:21.B"));
    // H colour is #60609f
    assertTrue(theCommand.contains("color #0/A:22 #60609f"));
    // V colour is #ffff00
    assertTrue(theCommand.contains("color #1/B:22 #ffff00"));
    // hidden columns are Gray (128, 128, 128)
    assertTrue(theCommand.contains("color #0/A:23-25|#1/B:23-25"));
    // S and G are both coloured #4949b6
    assertTrue(theCommand.contains("color #0/A:26-30|#1/B:26-30"));
  }
}
